#pragma once

#include "../../strings/format.h"
#include "../../strings/fileparts.h"

#include "../../exception.h"

#include "base.h"

#include <string>
#include <mutex>
#include <cstdio>

namespace debug::logger
{
	/* file logger class */
	class file : public base
	{
	public:
		file(const std::string& filename)
		{
			std::lock_guard<std::mutex> lock(this->m_lock);

			this->m_file = nullptr;

			fopen_s(&this->m_file, filename.c_str(), "wb+");

			if (this->m_file == nullptr)
				throw_exception(cannot_open_file_exception, filename);
		}

		~file(void)
		{
			std::lock_guard<std::mutex> lock(this->m_lock);

			if(this->m_file != nullptr)
				fclose(this->m_file);

			this->m_file = nullptr;
		}

		virtual void notify(const event& e) const override
		{
			std::lock_guard<std::mutex> lock(this->m_lock);

			if (this->m_file == nullptr)
				return;

			fprintf(this->m_file, "%s\t%s\t%s\t%s", e.get_string_time().c_str(), e.get_string_type().c_str(), e.get_filename().c_str(), e.get_message().c_str());

#ifdef _WIN32
			fputs("\r\n", this->m_file);
#else
			fputs("\n", this->m_file);
#endif

			fflush(this->m_file);
		}

	private:
		mutable std::mutex m_lock;

		FILE* m_file;
	};
};