#pragma once

#include "../exception.h"
#include "../log.h"

#include "../../exception.h"

#include "base.h"

#include <string>
#include <mutex>
#include <list>

namespace debug::logger
{
	/* hook logger class */
	class hook : public base
	{
	public:

		// add to list on notify
		virtual void notify(const event& e) const override
		{
			std::lock_guard<std::mutex> lock(this->m_lock);

			this->m_list.push_back(e);
		}

		// clear list
		void clear(void)
		{
			std::lock_guard<std::mutex> lock(this->m_lock);

			this->m_list.clear();
		}

		// retrieve complete list
		std::list<event> get(void) const
		{
			std::lock_guard<std::mutex> lock(this->m_lock);

			return this->m_list;
		}

		// filter list
		std::list<event> find(std::list<event::type> types, std::list<std::regex> filenames, std::list<std::regex> messages) const
		{
			std::list<event> ret;

			{
				std::lock_guard<std::mutex> lock(this->m_lock);

				// browse list
				for (auto& e : this->m_list)
				{
					// add if filter pass
					if(e.match(types, filenames, messages))
						ret.push_back(e);
				}
			}

			return ret;
		}

		// check if it has entry
		bool has(std::list<event::type> types, std::list<std::regex> filenames, std::list<std::regex> messages) const
		{
			std::lock_guard<std::mutex> lock(this->m_lock);

			// browse list
			for (auto& e : this->m_list)
			{
				// add if filter pass
				if (!e.match(types, filenames, messages))
					return false;
			}

			// successful
			return true;
		}

	private:
		mutable std::mutex m_lock;
		mutable std::list<event> m_list;
	};
};

using debug_logger_hook_t = debug::logger::auto_logger<debug::logger::hook>;
										