#include "manager.h"

#include "console.h"
#include "windbg.h"

#ifdef _WIN32
#include "evemon.h"
#endif

INITIALIZE_SINGLETON(debug_logger_manager_t);

debug::logger::manager::manager(void)
{
    // disable recursion lock
    {
        std::lock_guard<std::recursive_mutex> lk(this->m_lock);

        this->m_recurse_lock = false;
    }

    // add event monitoring in windows mode
#if defined(_WIN32) && defined(USE_EVEMON)
    add<logger::evemon>();
#endif

    // add console debuggers in _DEBUG mode
#ifdef _DEBUG

#ifdef _WIN32

#ifdef _CONSOLE
    add<logger::console>();
#else
    add<logger::windbg>();
#endif  

#else

    add<logger::console>();

#endif

#endif
}