#pragma once

#include "../types/type.h"
#include "../storage/ext/string.h"

#include "proxy.h"

namespace resources
{
	template<class type> class async_proxy : public proxy<type, true>, public storage::base
	{
	public:
		using proxy<type, true>::proxy;

		virtual std::string get_typename(void) const override
		{
			return decorate_typename("async_proxy<" + get_name_from_type<type>() + ">");
		}

		// push to storage object
		virtual void push(storage::fs::object_base& container) const
		{
			// create temporary object
			if (this->get() != nullptr)
			{
				storage::ext::string tmp(this->name().c_str());

				auto p = container.create_sub_object(get_typename(), "name");

				if(p != nullptr)
					tmp.push(*p);
			}

			// set typename
			container.set_type_name(get_typename());
		}

		// pop from storage object
		virtual void pop(const storage::fs::object_base& container)
		{
			// check typename
			if (!is_type_name(container.get_type_name(), get_typename()))
				throw_exception(wrong_type_exception, get_typename(), container.get_type_name());

			// check if "name" exists and load it
			auto p = container.get_sub_object(get_typename(), "name");

			if (p != nullptr)
			{
				storage::ext::string tmp;

				tmp.pop(*p);

				this->load(tmp);
			}
		}
	};
};